==> hlog-0.17.tar.gz: hlog-0.17/common/qso_serde_hlq.c <==
/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/buffer.h>
#include <jeffpc/nvl.h>
#include <jeffpc/error.h>

#include <hlog/version.h>

#include "qso_impl.h"
#include "hlq.h"

static const struct side_field {
	const char *name;
	struct val *(*get)(const struct qso_side *);
	int (*set)(struct qso_side *, struct val *);
} side_fields[] = {
	{ "station",   qso_get_station_call_val,  qso_set_station_call_val },
	{ "operator",  qso_get_operator_call_val, qso_set_operator_call_val },
	{ "power",     qso_get_power,             qso_set_power_val },
	{ "freq",      qso_get_freq,              qso_set_freq_val },
	{ "mode",      qso_get_mode_val,          qso_set_mode_val },
	{ "rst",       qso_get_rst_val,           qso_set_rst_val },
	{ "name",      qso_get_name_val,          qso_set_name_val },
	{ "qth",       qso_get_qth_val,           qso_set_qth_val },
	{ "grid",      qso_get_grid_val,          qso_set_grid_val },
	{ "country",   qso_get_country_val,       qso_set_country_val },
	{ "state",     qso_get_state_val,         qso_set_state_val },
	{ "county",    qso_get_county_val,        qso_set_county_val },
	{ "continent", qso_get_continent_val,     qso_set_continent_val },
	{ "dxcc",      qso_get_zone_dxcc,         qso_set_zone_dxcc_val },
	{ "itu",       qso_get_zone_itu,          qso_set_zone_itu_val },
	{ "cqz",       qso_get_zone_cqz,          qso_set_zone_cqz_val },
	{ "seq",       qso_get_seq,               qso_set_seq_val },
};

/*
 * Packing
 */

struct buffer *qso_pack_hlq(const struct qso *qso)
{
	const struct {
		enum hlq_ns ns;
		const struct qso_side *ptr;
	} sides[2] = {
		{ HLQ_NS_TX, &qso->tx },
		{ HLQ_NS_RX, &qso->rx },
	};
	const struct nvpair *pair;
	struct buffer *buf;
	size_t i, j;
	int ret;

	buf = buffer_alloc(1024);
	if (IS_ERR(buf))
		return buf;

	ret = hlq_append_header(buf, &qso->uuid);
	if (ret)
		goto err;

#define CHECK(expr) \
	do { \
		ret = (expr); \
		if (ret) \
			goto err; \
	} while (0)

#define _KV(buf, ns, k, v) \
	({ \
		struct val *val = (v); \
		int ret; \
		\
		ret = hlq_append_kv((buf), (ns), (k), val); \
		val_putref(val); \
		ret; \
	})
#define KV(buf, ns, k, v) CHECK(_KV(buf, ns, k, v))
#define KV_CSTR(buf, ns, k, v) CHECK(hlq_append_kv_cstr((buf), (ns), (k), (v)))
#define KV_TIME(buf, ns, k, v) CHECK(hlq_append_kv_time((buf), (ns), (k), (v)))

	KV_CSTR(buf, HLQ_NS_DEBUG, "hlog-version", version_string);

	KV_TIME(buf, HLQ_NS_QSO, "start", &qso->start);
	KV_TIME(buf, HLQ_NS_QSO, "end", &qso->end);
	KV(buf, HLQ_NS_QSO, "comment", qso_get_comment_val(qso));

	for (i = 0; i < ARRAY_LEN(sides); i++) {
		enum hlq_ns ns = sides[i].ns;
		const struct qso_side *side = sides[i].ptr;

		for (j = 0; j < ARRAY_LEN(side_fields); j++) {
			const struct side_field *f = &side_fields[j];

			KV(buf, ns, f->name, f->get(side));
		}
	}

	nvl_for_each(pair, qso->additional)
		KV(buf, HLQ_NS_ADDL, nvpair_name(pair), nvpair_value(pair));

#undef CHECK
#undef _KV
#undef KV
#undef KV_CSTR
#undef KV_TIME

	return buf;

err:
	buffer_free(buf);
	return ERR_PTR(ret);
}

/*
 * Unpacking
 */

static int __unpack_qso_ns(struct qso *qso, const char *key, struct val *val)
{
	int ret;

	if (!strcmp(key, "start")) {
		/* Note: qso_set_datetime_val with NULL will set it now */
		if (val)
			ret = qso_set_datetime_val(qso, true, val);
		else
			ret = 0;
	} else if (!strcmp(key, "end")) {
		/* Note: qso_set_datetime_val with NULL will set it now */
		if (val)
			ret = qso_set_datetime_val(qso, false, val);
		else
			ret = 0;
	} else if (!strcmp(key, "comment")) {
		ret = qso_set_comment_val(qso, val);
	} else {
		ret = -EINVAL;
	}

	return ret;
}

static int __unpack_side_ns(struct qso_side *side, const char *key,
			    struct val *val)
{
	size_t i;

	for (i = 0; i < ARRAY_LEN(side_fields); i++) {
		const struct side_field *f = &side_fields[i];

		if (strcmp(f->name, key))
			continue;

		return f->set(side, val);
	}

	return -ENOENT;
}

int qso_unpack_hlq(struct qso *qso, const void *raw, size_t len)
{
	struct buffer buf;
	struct xuuid uuid;
	int ret;

	buffer_init_static(&buf, raw, len, len, false);

	ret = hlq_parse_header(&buf, &uuid);
	if (ret)
		return ret;

	qso->uuid = uuid;
	xuuid_unparse(&qso->uuid, qso->uuid_str);

	while (buffer_remain(&buf)) {
		const char *key_start, *key_end;
		ptrdiff_t keylen;
		char key[32];
		struct val *val;
		enum hlq_ns ns;

		val = hlq_parse_kv(&buf, &ns, &key_start,
				   &key_end);
		if (IS_ERR(val))
			return PTR_ERR(val);

		/* extract & nul-terminate a key for ease of use */
		keylen = key_end - key_start;
		if (keylen >= sizeof(key)) {
			val_putref(val);
			return -ENAMETOOLONG;
		}

		memcpy(key, key_start, keylen);
		key[keylen] = '\0';

		switch (ns) {
			case HLQ_NS_DEBUG:
				val_putref(val); /* ignore value */
				break;
			case HLQ_NS_QSO:
				ret = __unpack_qso_ns(qso, key, val);
				break;
			case HLQ_NS_TX:
				ret = __unpack_side_ns(&qso->tx, key, val);
				break;
			case HLQ_NS_RX:
				ret = __unpack_side_ns(&qso->rx, key, val);
				break;
			case HLQ_NS_ADDL:
				ret = nvl_set(qso->additional, key, val);
				break;
		}

		if (ret)
			return ret;
	}

	return 0;
}

==> hlog-0.20-rc1.tar.gz: hlog-0.20-rc1/common/qso_serde_hlq.c <==
/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/buffer.h>
#include <jeffpc/nvl.h>
#include <jeffpc/error.h>

#include <hlog/version.h>

#include "qso_impl.h"
#include "hlq.h"

static const struct side_field {
	const char *name;
	struct val *(*get)(const struct qso_side *);
	int (*set)(struct qso_side *, struct val *);
} side_fields[] = {
	{ "station",   qso_get_station_call_val,  qso_set_station_call_val },
	{ "operator",  qso_get_operator_call_val, qso_set_operator_call_val },
	{ "power",     qso_get_power,             qso_set_power_val },
	{ "freq",      qso_get_freq,              qso_set_freq_val },
	{ "mode",      qso_get_mode_val,          qso_set_mode_val },
	{ "rst",       qso_get_rst_val,           qso_set_rst_val },
	{ "name",      qso_get_name_val,          qso_set_name_val },
	{ "qth",       qso_get_qth_val,           qso_set_qth_val },
	{ "grid",      qso_get_grid_val,          qso_set_grid_val },
	{ "country",   qso_get_country_val,       qso_set_country_val },
	{ "state",     qso_get_state_val,         qso_set_state_val },
	{ "county",    qso_get_county_val,        qso_set_county_val },
	{ "continent", qso_get_continent_val,     qso_set_continent_val },
	{ "dxcc",      qso_get_zone_dxcc,         qso_set_zone_dxcc_val },
	{ "itu",       qso_get_zone_itu,          qso_set_zone_itu_val },
	{ "cqz",       qso_get_zone_cqz,          qso_set_zone_cqz_val },
	{ "seq",       qso_get_seq,               qso_set_seq_val },
};

/*
 * Packing
 */

struct buffer *qso_pack_hlq(const struct qso *qso)
{
	const struct {
		enum qso_ns ns;
		const struct qso_side *ptr;
	} sides[2] = {
		{ QSO_NS_TX, &qso->tx },
		{ QSO_NS_RX, &qso->rx },
	};
	const struct nvpair *pair;
	struct buffer *buf;
	size_t i, j;
	int ret;

	buf = buffer_alloc(1024);
	if (IS_ERR(buf))
		return buf;

	ret = hlq_append_header(buf, &qso->uuid);
	if (ret)
		goto err;

#define CHECK(expr) \
	do { \
		ret = (expr); \
		if (ret) \
			goto err; \
	} while (0)

#define _KV(buf, ns, k, v) \
	({ \
		struct val *val = (v); \
		int ret; \
		\
		ret = hlq_append_kv((buf), (ns), (k), val); \
		val_putref(val); \
		ret; \
	})
#define KV(buf, ns, k, v) CHECK(_KV(buf, ns, k, v))
#define KV_CSTR(buf, ns, k, v) CHECK(hlq_append_kv_cstr((buf), (ns), (k), (v)))
#define KV_TIME(buf, ns, k, v) CHECK(hlq_append_kv_time((buf), (ns), (k), (v)))

	KV_CSTR(buf, QSO_NS_DEBUG, "hlog-version", version_string);

	KV_TIME(buf, QSO_NS_QSO, "start", &qso->start);
	KV_TIME(buf, QSO_NS_QSO, "end", &qso->end);
	KV(buf, QSO_NS_QSO, "comment", qso_get_comment_val(qso));

	for (i = 0; i < ARRAY_LEN(sides); i++) {
		enum qso_ns ns = sides[i].ns;
		const struct qso_side *side = sides[i].ptr;

		for (j = 0; j < ARRAY_LEN(side_fields); j++) {
			const struct side_field *f = &side_fields[j];

			KV(buf, ns, f->name, f->get(side));
		}
	}

	nvl_for_each(pair, qso->additional)
		KV(buf, QSO_NS_ADDL, nvpair_name(pair), nvpair_value(pair));

#undef CHECK
#undef _KV
#undef KV
#undef KV_CSTR
#undef KV_TIME

	return buf;

err:
	buffer_free(buf);
	return ERR_PTR(ret);
}

/*
 * Unpacking
 */

static int __unpack_qso_ns(struct qso *qso, const char *key, struct val *val)
{
	int ret;

	if (!strcmp(key, "start")) {
		/* Note: qso_set_datetime_val with NULL will set it now */
		if (val)
			ret = qso_set_datetime_val(qso, true, val);
		else
			ret = 0;
	} else if (!strcmp(key, "end")) {
		/* Note: qso_set_datetime_val with NULL will set it now */
		if (val)
			ret = qso_set_datetime_val(qso, false, val);
		else
			ret = 0;
	} else if (!strcmp(key, "comment")) {
		ret = qso_set_comment_val(qso, val);
	} else {
		ret = -EINVAL;
	}

	return ret;
}

static int __unpack_side_ns(struct qso_side *side, const char *key,
			    struct val *val)
{
	size_t i;

	for (i = 0; i < ARRAY_LEN(side_fields); i++) {
		const struct side_field *f = &side_fields[i];

		if (strcmp(f->name, key))
			continue;

		return f->set(side, val);
	}

	return -ENOENT;
}

int qso_set_hlq_item(struct qso *qso, enum qso_ns ns, const char *key_start,
		     const char *key_end, struct val *val)
{
	ptrdiff_t keylen;
	char key[32];
	int ret;

	/* extract & nul-terminate a key for ease of use */
	keylen = key_end - key_start;
	if (keylen >= sizeof(key)) {
		val_putref(val);
		return -ENAMETOOLONG;
	}

	memcpy(key, key_start, keylen);
	key[keylen] = '\0';

	switch ((enum qso_ns) ns) {
		case QSO_NS_DEBUG:
			val_putref(val); /* ignore value */
			ret = 0;
			break;
		case QSO_NS_QSO:
			ret = __unpack_qso_ns(qso, key, val);
			break;
		case QSO_NS_TX:
			ret = __unpack_side_ns(&qso->tx, key, val);
			break;
		case QSO_NS_RX:
			ret = __unpack_side_ns(&qso->rx, key, val);
			break;
		case QSO_NS_ADDL:
			ret = nvl_set(qso->additional, key, val);
			break;
	}

	return ret;
}

int qso_unpack_hlq(struct qso *qso, const void *raw, size_t len)
{
	struct buffer buf;
	struct xuuid uuid;
	int ret;

	buffer_init_static(&buf, raw, len, len, false);

	ret = hlq_parse_header(&buf, &uuid);
	if (ret)
		return ret;

	qso->uuid = uuid;
	xuuid_unparse(&qso->uuid, qso->uuid_str);

	while (buffer_remain(&buf)) {
		const char *key_start, *key_end;
		struct val *val;
		enum qso_ns ns;

		val = hlq_parse_kv(&buf, &ns, &key_start,
				   &key_end);
		if (IS_ERR(val))
			return PTR_ERR(val);

		ret = qso_set_hlq_item(qso, ns, key_start, key_end, val);
		if (ret)
			return ret;
	}

	return 0;
}
